from tsuru_dashboard import settings
from tsuru_dashboard.metrics.backends import elasticsearch, prometheus
from tsuru_dashboard.metrics.backends import base

import requests


def get_app(app_name, token):
    url = '{}/apps/{}'.format(settings.TSURU_HOST, app_name)
    headers = {"authorization": token}
    return requests.get(url, headers=headers).json()


def get_envs(app_name, token):
    url = '{}/apps/{}/env'.format(settings.TSURU_HOST, app_name)
    headers = {"authorization": token}
    data = requests.get(url, headers=headers).json()
    envs = {}
    for env in data:
        envs[env['name']] = env['value']

    return envs


def get_app_backend(app_name, token, **kwargs):
    app = get_app(app_name, token)
    envs = base.get_envs_from_api(app, token)
    url = ""
    backends = []

    if envs and "METRICS_PROMETHEUS_HOST" in envs:
        url = envs["METRICS_PROMETHEUS_HOST"]
        backends.append(prometheus.AppBackend(app=app, url=url, **kwargs))

    if envs and "METRICS_ELASTICSEARCH_HOST" in envs:
        url = envs["METRICS_ELASTICSEARCH_HOST"]
        backends.append(elasticsearch.AppBackend(app=app, url=url, **kwargs))

    if settings.PROMETHEUS_HOST:
        url = settings.PROMETHEUS_HOST
        backends.append(prometheus.AppBackend(app=app, url=url, **kwargs))

    if settings.ELASTICSEARCH_HOST:
        url = settings.ELASTICSEARCH_HOST
        backends.append(elasticsearch.AppBackend(app=app, url=url, **kwargs))

    if not url:
        app["envs"] = get_envs(app_name, token)
        if "envs" in app and "ELASTICSEARCH_HOST" in app["envs"]:
            url = app["envs"]["ELASTICSEARCH_HOST"]
            backends.append(elasticsearch.AppBackend(app=app, url=url, **kwargs))

    return backends


def get_tsuru_backend(component, token, **kwargs):
    date_range = kwargs["date_range"]
    component_filter = elasticsearch.ComponentFilter(
        component=component,
        date_range=date_range
    )
    return elasticsearch.TsuruMetricsBackend(
        url=settings.ELASTICSEARCH_HOST,
        filter=component_filter,
        date_range=date_range,
    )



from django.test import TestCase
from tsuru_dashboard.metrics.backends import base
from mock import patch, Mock
from tsuru_dashboard import settings


class BackendTest(TestCase):
    @patch("requests.get")
    def test_envs_from_api(self, get_mock):
        data = {
            "METRICS_BACKEND": "logstash",
            "METRICS_ELASTICSEARCH_HOST": "http://easearch.com",
            "METRICS_LOGSTASH_HOST": "logstash.com"
        }
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = data
        get_mock.return_value = response_mock

        app = {"name": "appname", "units": [{"ProcessName": "web"}]}

        envs = base.get_envs_from_api(app, 'token')
        self.assertDictEqual(envs, data)

    def test_set_destination_hostname(self):
        tests = [
            ("", ""),
            ("x", "x"),
            ("127.0.0.9", "127.0.0.9"),
            ("127.0.0.9:1234", "127.0.0.9:1234"),
            ("127.0.0.1", "127.0.0.1(localhost)"),
            ("127.0.0.1:1234", "127.0.0.1:1234(localhost)"),
        ]
        settings.RESOLVE_CONNECTION_HOSTS = False
        for v1, v2 in tests:
            self.assertEqual(base.set_destination_hostname(v1), v1)
        settings.RESOLVE_CONNECTION_HOSTS = True
        for v1, v2 in tests:
            self.assertEqual(base.set_destination_hostname(v1), v2)
        settings.RESOLVE_CONNECTION_HOSTS = False



from django.http import HttpResponse, HttpResponseRedirect
from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse
from django.views.generic.base import View

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredMixin

from mock import patch, Mock


class LoginRequiredMixinTest(TestCase):

    @patch("requests.get")
    def test_should_redirect_to_login_page_if_user_is_not_authenticated(self, get):
        get.return_value = Mock(status_code=200)
        request = RequestFactory().get('/')
        request.session = {}
        response = StubView.as_view()(request)
        self.assertIsInstance(response, HttpResponseRedirect)
        expected_url = "%s?next=%s" % (reverse('login'), request.path)
        self.assertEqual(expected_url, response['Location'])

    @patch("requests.get")
    def test_should_invoke_view_dispatch_when_user_is_authenticated(self, get):
        get.return_value = Mock(status_code=200)
        request = RequestFactory().get('/')
        request.session = {'tsuru_token': 'my beautiful token'}
        response = StubView.as_view()(request)
        self.assertEqual('ok', response.content)

    def test_authorization(self):
        view = StubView()

        request = RequestFactory().get('/')
        request.session = {'tsuru_token': 'my beautiful token'}

        view.request = request
        headers = view.authorization

        expected = {'authorization': 'my beautiful token'}
        self.assertDictEqual(headers, expected)

    def test_client(self):
        view = StubView()

        request = RequestFactory().get('/')
        request.session = {'tsuru_token': 'type mytoken'}

        view.request = request
        client = view.client

        self.assertEqual(client.templates.target, settings.TSURU_HOST)
        self.assertEqual(client.templates.token, "mytoken")


class StubView(LoginRequiredMixin, View):

    def get(self, request):
        return HttpResponse('ok')



from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.teams.views import Info

from mock import patch, Mock


class InfoViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}
        self.data = {u'name': u'anvenger', u'users': [u'ironman', u'thor']}

    @patch("requests.get")
    def test_view(self, get):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = self.data
        get.return_value = response_mock

        response = Info.as_view()(self.request, team="avengers")

        self.assertIn("teams/info.html", response.template_name)
        self.assertDictEqual(self.data, response.context_data['team'])
        get.assert_called_with(
            '{0}/teams/{1}'.format(settings.TSURU_HOST, "avengers"),
            headers={'authorization': 'admin'}
        )



from mock import patch

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard.apps.views import AppLog


class AppLogViewTest(TestCase):
    @patch("requests.get")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def setUp(self, token_is_valid, get):
        token_is_valid.return_value = True
        self.request = RequestFactory().get("/apps/app-teste/log/")
        self.request.session = {'tsuru_token': 'tokentest'}
        self.app_name = 'app-teste'
        self.response = AppLog.as_view()(self.request, app_name=self.app_name)

    def test_run_should_render_expected_template(self):
        self.assertIn('apps/app_log.html', self.response.template_name)

    def test_context_should_contain_app(self):
        self.assertIn('app', self.response.context_data)

    def test_get_request_run_url_should_not_return_404(self):
        response = self.client.get(reverse('app_log', args=[self.app_name]))
        self.assertNotEqual(404, response.status_code)



from tsuru_dashboard import settings
from tsuru_dashboard.metrics.backends import base

import requests
import json
import datetime


NET_AGGREGATION = {
    "units": {
        "terms": {
            "field": "host.keyword"
        },
        "aggs": {
            "delta": {
                "scripted_metric": {
                    "init_script": {
                        "lang": "groovy",
                        "inline": """
_agg['max'] = [ts: 0, val: null]
_agg['min'] = [ts: 0, val: null]
"""},
                    "map_script": {
                        "lang": "groovy",
                        "inline": """
ts = doc['@timestamp'][0]
val = doc['value'][0]
if (ts > _agg.max.ts) {
    _agg.max.ts = ts
    _agg.max.val = val
}
if (_agg.min.ts == 0 || ts < _agg.min.ts) {
    _agg.min.ts = ts
    _agg.min.val = val
}
"""},
                    "reduce_script": {
                        "lang": "groovy",
                        "inline": """
max = null
min = null
for (a in _aggs) {
    if (max == null || a.max.ts > max.ts) {
        max = a.max
    }
    if (min == null || a.min.ts < min.ts) {
        min = a.min
    }
}
dt = max.ts - min.ts
if (dt > 0) {
    return ((max.val - min.val)/1024)/(dt/1000)
} else {
    return 0
}
"""}
                }
            }
        }
    }
}


class ElasticSearchFilter(object):
    def query(self):
        return self.filter

    def term_filter(self, field, value):
        return {"term": {field: value}}

    def terms_filter(self, field, values):
        if(type(values) is not list):
            values = [values]
        return {"terms": {field: values}}

    def timestamp_filter(self, date_range):
        if date_range is None:
            date_range = "1h"
        return {"range": {"@timestamp": {"gte": "now-" + date_range, "lt": "now"}}}

    def metric_filter(self, *filters):
        bool_filter = {
            "bool": {
                "must": [
                    self.timestamp_filter(self.date_range)
                ]
            }
        }
        if list(filters):
            bool_filter["bool"]["must"].append({"bool": {"should": list(filters)}})

        return bool_filter


class NodeFilter(ElasticSearchFilter):
    def __init__(self, node, date_range=None):
        self.date_range = date_range
        self.filter = self.node_filter(node)

    def node_filter(self, node):
        return self.metric_filter(self.terms_filter("addr.keyword", node))


class ComponentFilter(ElasticSearchFilter):
    def __init__(self, component, date_range=None):
        self.date_range = date_range
        self.filter = self.component_filter(component)

    def component_filter(self, component):
        f = self.metric_filter()
        f["bool"]["must"].append(self.term_filter("container.keyword", component))
        return f


class AppFilter(ElasticSearchFilter):
    def __init__(self, app, process_name=None, date_range=None):
        self.date_range = date_range
        self.filter = self.app_filter(app, process_name)

    def app_filter(self, app, process_name):
        f = self.metric_filter()
        f["bool"]["must"].append(self.term_filter("app.keyword", app))

        if process_name:
            p = self.term_filter("process.keyword", process_name)
            f["bool"]["must"].append(p)
        return f


class ElasticSearch(object):
    def __init__(self, url, query, date_range="1h"):
        if date_range is None:
            date_range = "1h"
        if date_range == "1h":
            self.index = "{}-{}*".format(settings.ELASTICSEARCH_INDEX,
                                         datetime.datetime.utcnow().strftime("%Y.%m.%d"))
        else:
            self.index = "{}-{}.*".format(settings.ELASTICSEARCH_INDEX,
                                          datetime.datetime.utcnow().strftime("%Y"))
        self.url = url
        self.filtered_query = query
        self.date_range = date_range

    def post(self, data, metric):
        url = "{}/{}/{}/_search".format(self.url, self.index, metric)
        result = requests.post(url, data=json.dumps(data))
        return result.json()

    def process(self, data, formatter=None):
        if not formatter:
            def default_formatter(x):
                return x
            formatter = default_formatter

        def processor(result, bucket):
            bucket_max = formatter(bucket["stats"]["max"])
            bucket_min = formatter(bucket["stats"]["min"])
            bucket_avg = formatter(bucket["stats"]["avg"])
            if not result:
                result = {
                    "max": [],
                    "min": [],
                    "avg": [],
                }
            result["max"].append([bucket["key"], bucket_max])
            result["min"].append([bucket["key"], bucket_min])
            result["avg"].append([bucket["key"], bucket_avg])
            return result, bucket_min, bucket_max

        return self.base_process(data, processor)

    def cpu_max(self, interval=None):
        query = self.query(interval=interval)
        query["query"]["bool"]["must"].append(
            {"range": {"value": {"lt": 500}}}
        )
        response = self.post(query, "cpu_max")
        process = self.process(response)
        return process

    def mem_max(self, interval=None):
        query = self.query(interval=interval)
        return self.process(self.post(query, "mem_max"), formatter=lambda x: x / (1024 * 1024))

    def swap(self, interval=None):
        query = self.query(interval=interval)
        return self.process(self.post(query, "swap"), formatter=lambda x: x / (1024 * 1024))

    def netrx(self, interval=None):
        return self.net_metric("netrx", interval)

    def nettx(self, interval=None):
        return self.net_metric("nettx", interval)

    def net_metric(self, kind, interval=None):
        query = self.query(interval=interval, aggregation=NET_AGGREGATION)
        return self.base_process(self.post(query, kind), self.net_process)

    def net_process(self, result, bucket):
        value = 0
        for in_bucket in bucket["units"]["buckets"]:
            value += in_bucket["delta"]["value"]
        if not result:
            result["requests"] = []
        result["requests"].append([bucket["key"], value])
        return result, value, value

    def units(self, interval=None):
        aggregation = {"units": {"cardinality": {"field": "host.keyword"}}}
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "cpu_max"), self.units_process)

    def units_process(self, result, bucket):
        value = bucket["units"]["value"]
        if not result:
            result["units"] = []
        result["units"].append([bucket["key"], value])
        return result, value, value

    def requests_min(self, interval=None):
        aggregation = {"sum": {"sum": {"field": "count"}}}
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "response_time"), self.requests_min_process)

    def requests_min_process(self, result, bucket):
        value = bucket["sum"]["value"]
        if not result:
            result["requests"] = []
        result["requests"].append([bucket["key"], value])
        return result, value, value

    def response_time(self, interval=None):
        aggregation = {
            "stats": {"stats": {"field": "value"}},
            "percentiles": {"percentiles": {"field": "value"}}
        }
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "response_time"), self.response_time_process)

    def response_time_process(self, result, bucket):
        bucket_max = bucket["stats"]["max"]
        bucket_min = bucket["stats"]["min"]
        bucket_avg = bucket["stats"]["avg"]
        if not result:
            result = {
                "max": [],
                "min": [],
                "avg": [],
                "p95": [],
                "p99": [],
            }
        result["max"].append([bucket["key"], bucket_max])
        result["min"].append([bucket["key"], bucket_min])
        result["avg"].append([bucket["key"], bucket_avg])
        result["p95"].append([bucket["key"], bucket["percentiles"]["values"]["95.0"]])
        result["p99"].append([bucket["key"], bucket["percentiles"]["values"]["99.0"]])
        return result, bucket_min, bucket_max

    def top_slow(self, interval=None):
        query = {
            "query": self.filtered_query,
            "size": 0,
            "aggs": {
                "top": {
                    "terms": {
                        "script": "doc['method'].value +'|-o-|'+doc['path.keyword'].value +'|-o-|'+doc['status_code'].value",
                        "order": {
                            "stats.max": "desc"
                        }
                    },
                    "aggs": {
                        "stats": {"stats": {"field": "value"}},
                        "percentiles": {"percentiles": {"field": "value"}},
                        "max": {
                            "top_hits": {
                                "sort": [{"value": {"order": "desc"}}],
                                "size": 1
                            }
                        }
                    }
                }
            }
        }
        return self.top_slow_process(self.post(query, "response_time"))

    def top_slow_process(self, data):
        buckets = []
        result = []
        if "aggregations" in data and len(data["aggregations"]["top"]["buckets"]) > 0:
            buckets = data["aggregations"]["top"]["buckets"]
        for b in buckets:
            parts = b["key"].split("|-o-|")
            if len(parts) != 3:
                continue
            method = parts[0]
            path = parts[1]
            status_code = parts[2]

            last_ts = b["max"]["hits"]["hits"][0]["_source"]["@timestamp"]
            result.append({
                "last_time": last_ts,
                "stats": b["stats"],
                "percentiles": b["percentiles"]["values"],
                "method": method,
                "path": path,
                "status_code": status_code,
            })

        return result

    def http_methods(self, interval=None):
        aggregation = {"method": {"terms": {"field": "method"}}}
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "response_time"), self.http_methods_process)

    def http_methods_process(self, result, bucket):
        max_value = 0
        min_value = 0

        for doc in bucket["method"]["buckets"]:
            value = doc["doc_count"]
            method = doc["key"]

            if method not in result:
                result[method] = []

            if value < min_value:
                min_value = value

            if value > max_value:
                max_value = value

            result[method].append([bucket["key"], value])
        return result, min_value, max_value

    def status_code(self, interval=None):
        aggregation = {"status_code": {"terms": {"field": "status_code"}}}
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "response_time"), self.status_code_process)

    def status_code_process(self, result, bucket):
        max_value = 0
        min_value = 0

        for doc in bucket["status_code"]["buckets"]:
            value = doc["doc_count"]
            code = doc["key"]

            if code not in result:
                result[code] = []

            if value < min_value:
                min_value = value

            if value > max_value:
                max_value = value

            result[code].append([bucket["key"], value])
        return result, min_value, max_value

    def connections(self, interval=None):
        aggregation = {
            "connection": {
                "terms": {"field": "connection.keyword"}
            }
        }
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "connection"), self.connections_process)

    def connections_process(self, result, bucket):
        min_value = 0
        max_value = 0
        for doc in bucket["connection"]["buckets"]:
            size = doc["doc_count"]
            conn = base.set_destination_hostname(doc["key"])
            if conn not in result:
                result[conn] = []

            if size < min_value:
                min_value = size

            if size > max_value:
                max_value = size

            result[conn].append([bucket["key"], size])
        return result, min_value, max_value

    def base_process(self, data, processor):
        result = {}
        min_value = None
        max_value = 0

        if "aggregations" in data and len(data["aggregations"]["date"]["buckets"]) > 0:
            for bucket in data["aggregations"]["date"]["buckets"]:
                result, min_v, max_v = processor(result, bucket)
                if min_value is None or min_v < min_value:
                    min_value = min_v
                if max_v > max_value:
                    max_value = max_v

        return {
            "data": result,
            "min": min_value or 0,
            "max": max_value + 1,
        }

    def query(self, interval=None, aggregation=None):
        if not interval:
            interval = "1m"

        if not aggregation:
            aggregation = {"stats": {"stats": {"field": "value"}}}
        return {
            "query": self.filtered_query,
            "size": 0,
            "aggs": {
                "date": {
                    "date_histogram": {
                        "field": "@timestamp",
                        "interval": interval
                    },
                    "aggs": aggregation,
                }
            }
        }


class AppBackend(ElasticSearch):
    def __init__(self, app, url, process_name=None, date_range=None):
        es_query = AppFilter(app=app["name"], process_name=process_name, date_range=date_range).query()
        return super(AppBackend, self).__init__(
            url=url,
            query=es_query,
            date_range=date_range
        )


class TsuruMetricsBackend(ElasticSearch):
    def __init__(self, filter, url=None, date_range=None):
        if not url:
            url = settings.ELASTICSEARCH_HOST
        return super(TsuruMetricsBackend, self).__init__(
            url=url, query=filter.query(), date_range=date_range)


class NodeMetricsBackend(TsuruMetricsBackend):
    def __init__(self, addr, date_range=None):
        filter = NodeFilter(node=addr, date_range=date_range)
        return super(NodeMetricsBackend, self).__init__(filter=filter, date_range=date_range)

    def multi_index_avg(self, result, bucket, formatter=None):
        if formatter is None:
            def formatter(x):
                return x
        for b in bucket["stats"]["buckets"]:
            result[b["key"].split('_')[-1]].append([bucket["key"], formatter(b["stats"]["avg"])])
        return result, None, None

    def load_process(self, result, bucket):
        if not result:
            result = {
                "load1": [],
                "load5": [],
                "load15": [],
            }
        return self.multi_index_avg(result, bucket)

    def cpu_max_process(self, result, bucket):
        if not result:
            result = {
                "user": [],
                "sys": [],
                "wait": [],
            }
        return self.multi_index_avg(result, bucket, formatter=lambda x: x * 100)

    def disk_process(self, result, bucket):
        if not result:
            result = {
                "used": [],
                "total": []
            }
        return self.multi_index_avg(result, bucket, formatter=lambda x: x / (1024 * 1024))

    def per_type_agg(self):
        return {
            "stats": {
                "terms": {"field": "_type"},
                "aggs": {"stats": {"stats": {"field": "value"}}}
            }
        }

    def load(self, interval=None):
        query = self.query(interval=interval, aggregation=self.per_type_agg())
        return self.base_process(self.post(query, "host_load1,host_load5,host_load15"), self.load_process)

    def cpu_max(self, interval=None):
        query = self.query(interval=interval, aggregation=self.per_type_agg())
        process = self.base_process(self.post(
            query, "host_cpu_user,host_cpu_sys,host_cpu_wait"), self.cpu_max_process)
        return process

    def mem_max(self, interval=None):
        query = self.query(interval=interval)
        return self.process(self.post(query, "host_mem_used"), formatter=lambda x: x / (1024 * 1024))

    def netrx(self, interval=None):
        return self.net_metric("host_netrx", interval)

    def nettx(self, interval=None):
        return self.net_metric("host_nettx", interval)

    def swap(self, interval=None):
        query = self.query(interval=interval, aggregation=self.per_type_agg())
        return self.base_process(self.post(query, "host_swap_used,host_swap_total"), self.disk_process)

    def disk(self, interval=None):
        query = self.query(interval=interval, aggregation=self.per_type_agg())
        return self.base_process(self.post(query, "host_disk_used,host_disk_total"), self.disk_process)


class NodesMetricsBackend(TsuruMetricsBackend):
    def __init__(self, addrs, date_range=None):
        self.addrs = addrs
        filter = NodeFilter(node=addrs, date_range=date_range)
        return super(NodesMetricsBackend, self).__init__(filter=filter, date_range=date_range)

    def process(self, data, formatter=None, processor=None):
        if formatter is None:
            def default_formatter(x):
                return x
            formatter = default_formatter
        if processor is None:
            def default_processor(result, bucket):
                if not result:
                    result = {}
                    for addr in self.addrs:
                        result[addr] = []

                for b in bucket["addrs"]["buckets"]:
                    result[b["key"]].append([bucket["key"], formatter(b["avg"]["value"])])
                return result, None, None
            processor = default_processor
        return self.base_process(data=data, processor=processor)

    def per_addr_agg(self, aggs=None):
        if aggs is None:
            aggs = {"avg": {"avg": {"field": "value"}}}
        return {
            "addrs": {
                "terms": {
                    "field": "addr.keyword",
                    "include": '|'.join(self.addrs),
                    "size": len(self.addrs)
                },
                "aggs": aggs
            }
        }

    def mem_max(self, interval=None):
        query = self.query(interval=interval, aggregation=self.per_addr_agg())
        return self.process(self.post(query, "host_mem_used"), formatter=lambda x: x / (1024 * 1024))

    def cpu_max(self, interval=None):
        query = self.query(interval=interval, aggregation=self.per_addr_agg())
        return self.process(self.post(query, "host_cpu_busy"), formatter=lambda x: x * 100)

    def cpu_wait(self, interval=None):
        query = self.query(interval=interval, aggregation=self.per_addr_agg())
        return self.process(self.post(query, "host_cpu_wait"), formatter=lambda x: x * 100)

    def swap(self, interval=None):
        query = self.query(interval=interval, aggregation=self.per_addr_agg())
        return self.process(self.post(query, "host_swap_used"), formatter=lambda x: x / (1024 * 1024))

    def disk(self, interval=None):
        query = self.query(interval=interval, aggregation=self.per_addr_agg())
        return self.process(self.post(query, "host_disk_used"), formatter=lambda x: x / (1024 * 1024))

    def netrx(self, interval=None):
        query = self.query(interval=interval, aggregation=self.per_addr_agg(
            aggs=NET_AGGREGATION["units"]["aggs"]))
        return self.process(self.post(query, "host_netrx"), processor=self.net_processor)

    def nettx(self, interval=None):
        query = self.query(interval=interval, aggregation=self.per_addr_agg(
            aggs=NET_AGGREGATION["units"]["aggs"]))
        return self.process(self.post(query, "host_nettx"), processor=self.net_processor)

    def load1(self, interval=None):
        return self.load(mins=1, interval=interval)

    def load5(self, interval=None):
        return self.load(mins=5, interval=interval)

    def load15(self, interval=None):
        return self.load(mins=15, interval=interval)

    def load(self, mins, interval=None):
        query = self.query(interval=interval, aggregation=self.per_addr_agg())
        return self.process(self.post(query, "host_load" + str(mins)))

    def net_processor(self, result, bucket):
        if not result:
            result = {}
            for addr in self.addrs:
                result[addr] = []
        for b in bucket["addrs"]["buckets"]:
            result[b["key"]].append([bucket["key"], b["delta"]["value"]])
        return result, None, None



import json
import httpretty
import mock

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.admin.views import ListDeploy


class ListDeployViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/?page=2")
        self.request.session = {"tsuru_token": "admin"}

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_use_list_template(self, token_is_valid):
        token_is_valid.return_value = True

        url = '{}/deploys?skip=0&limit=20'.format(settings.TSURU_HOST)
        body = json.dumps(range(10000))
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        response = ListDeploy.as_view()(self.request)

        self.assertIn("deploys/list_deploys.html", response.template_name)
        self.assertIn('deploys', response.context_data.keys())
        self.assertEqual(3, response.context_data["next"])
        self.assertEqual(1, response.context_data["previous"])

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_return_empty_list_when_status_is_204(self, token_is_valid):
        token_is_valid.return_value = True

        url = '{}/deploys?skip=0&limit=20'.format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, url, status=204)

        response = ListDeploy.as_view()(self.request)

        self.assertIn("deploys/list_deploys.html", response.template_name)
        self.assertListEqual([], response.context_data['deploys'])



from mock import patch, Mock
from base64 import encodestring
import os.path
import subprocess

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.apps.views import ListDeploy


class ListDeployViewTest(TestCase):
    @patch('requests.get')
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def setUp(self, token_is_valid, get):
        token_is_valid.return_value = True
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}
        self.response = ListDeploy.as_view()(self.request, app_name="appname")

    def test_context_should_contain_app(self):
        self.assertIn('app', self.response.context_data.keys())

    @patch('requests.get')
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_use_deploys_template(self, token_is_valid, get):
        token_is_valid.return_value = True
        get.return_value = Mock(status_code=200)
        self.assertEqual("apps/deploys.html", self.response.template_name)
        self.assertIn('deploys', self.response.context_data.keys())

    @patch('requests.get')
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_deploy_list(self, token_is_valid, get):
        token_is_valid.return_value = True
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}

        view = ListDeploy
        view.get_app = Mock()
        self.response = view.as_view()(self.request, app_name="appname")

        url = '{}/deploys?app=appname&skip=0&limit=20'.format(settings.TSURU_HOST)
        headers = {'authorization': 'admin'}
        get.assert_called_with(url, headers=headers)

    @patch('requests.get')
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_empty_list(self, token_is_valid, get):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = None
        get.return_value = response_mock
        token_is_valid.return_value = True

        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}

        view = ListDeploy
        view.get_app = Mock()
        view.as_view()(request, app_name="appname")

        url = '{}/deploys?app=appname&skip=0&limit=20'.format(settings.TSURU_HOST)
        headers = {'authorization': 'admin'}
        get.assert_called_with(url, headers=headers)

    @patch('requests.post')
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_targz(self, token_is_valid, post):
        response_mock = Mock(status_code=200)
        response_mock.iter_lines.return_value = iter(["expected"])

        post.return_value = response_mock

        token_is_valid.return_value = True

        BASE_DIR = os.path.abspath(os.path.dirname(__file__))
        zip_file = open(os.path.join(BASE_DIR, "example.zip"))
        encoded = encodestring(zip_file.read())

        request = RequestFactory().post("/", {"filecontent": encoded})
        request.session = {"tsuru_token": "admin"}
        tar_path = os.path.join(BASE_DIR, "example.tar.gz")

        def deploy(self, request, app_name, content):
            tar_file = open(tar_path, "wb")
            tar_file.write(content.getvalue())
            tar_file.close()

        view = ListDeploy
        view.deploy = deploy
        view.as_view()(request, app_name="appname")

        cmd = ["tar", "zxvf", "{}".format(tar_path), "-C", "/tmp"]
        pipes = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        _, stderr = pipes.communicate()

        self.assertEqual(pipes.returncode, 0)
        self.assertNotIn("Damaged", stderr)



from django.test import TestCase
from django.test.client import RequestFactory
from django.http import Http404

from tsuru_dashboard.apps.views import Settings

from mock import patch, Mock


class AppSettingsTestCase(TestCase):
    @patch("requests.get")
    def setUp(self, requests_mock):
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}
        self.expected = {
            "name": "app1",
            "framework": "php",
            "repository": "git@git.com:php.git",
            "state": "dead",
            "units": [
                {"Ip": "10.10.10.10", "Status": "started", "ProcessName": "web"},
                {"Ip": "9.9.9.9", "Status": "stopped", "ProcessName": "worker"},
            ],
            "tags": ["tag 1", "tag 2"],
            "teams": ["tsuruteam", "crane"]
        }
        json_mock = Mock(status_code=200)
        json_mock.json.return_value = self.expected
        requests_mock.return_value = json_mock

        self.response = Settings.as_view()(request, app_name="app1")
        self.request = request

    def test_should_use_settings_template(self):
        self.assertIn("apps/settings.html", self.response.template_name)

    def test_should_get_the_app_info_from_tsuru(self):
        self.assertDictEqual(self.expected, self.response.context_data["app"])

    @patch("requests.get")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_not_found(self, token_is_valid, requests_mock):
        token_is_valid.return_value = True
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}
        requests_mock.return_value = Mock(status_code=404)

        with self.assertRaises(Http404):
            Settings.as_view()(request, app_name="app1")

    @patch("requests.get")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_app_tags(self, token_is_valid, requests_mock):
        token_is_valid.return_value = True
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}
        response_mock = Mock(status_code=200)
        requests_mock.return_value = response_mock

        # Not set
        response_mock.json.return_value = {}
        response = Settings.as_view()(request, app_name="app1")
        self.assertEqual("", response.context_data["app"]["tags"])

        # Empty string
        response_mock.json.return_value = {"tags": ""}
        response = Settings.as_view()(request, app_name="app1")
        self.assertEqual("", response.context_data["app"]["tags"])

        # Empty array
        response_mock.json.return_value = {"tags": []}
        response = Settings.as_view()(request, app_name="app1")
        self.assertEqual("", response.context_data["app"]["tags"])

        # None
        response_mock.json.return_value = {"tags": None}
        response = Settings.as_view()(request, app_name="app1")
        self.assertEqual("", response.context_data["app"]["tags"])

        # Valid tags
        response_mock.json.return_value = {"tags": ["tag1", "tag2"]}
        response = Settings.as_view()(request, app_name="app1")
        self.assertEqual("tag1, tag2", response.context_data["app"]["tags"])



from django.conf.urls import url

import views


urlpatterns = [
    url(r'^$', views.ListComponent.as_view(), name='list-component'),
    url(r'^list.json$', views.ListComponentJson.as_view(), name='list-component-json')
]



from django.test import TestCase

from tsuru_dashboard.auth import forms


class SignupFormTest(TestCase):
    def test_fields(self):
        self.assertIn('email', forms.SignupForm.base_fields)
        self.assertIn('password', forms.SignupForm.base_fields)
        self.assertIn('same_password_again', forms.SignupForm.base_fields)

    def test_password_confirmation_is_valid(self):
        data = {'email': 'test@test.com', 'password': 'abc123',
                'same_password_again': 'abc123'}
        form = forms.SignupForm(data)
        self.assertTrue(form.is_valid())

    def test_password_confirmation_is_invalid(self):
        data = {'email': 'test@test.com', 'password': 'abc123',
                'same_password_again': 'not the same'}
        form = forms.SignupForm(data)
        self.assertFalse(form.is_valid())
        self.assertIn(u'You must type the same password twice!',
                      form.errors['same_password_again'])



from django.test import TestCase
from tsuru_dashboard.metrics.backends.elasticsearch import ElasticSearch, AppFilter, NET_AGGREGATION
from mock import patch, Mock
import datetime
import json


class ElasticSearchTest(TestCase):
    def setUp(self):
        self.maxDiff = None
        self.es = ElasticSearch("http://url.com", AppFilter(app="app_name").query())
        self.index = ".measure-tsuru-{}*".format(datetime.datetime.utcnow().strftime("%Y.%m.%d"))

    @patch("requests.post")
    def test_cpu_max(self, post_mock):
        self.es.process = Mock()
        self.es.cpu_max()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "cpu_max")
        post_mock.assert_called_with(url, data=json.dumps(self.es.query()))

    @patch("requests.post")
    def test_mem_max(self, post_mock):
        self.es.process = Mock()
        self.es.mem_max()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "mem_max")
        post_mock.assert_called_with(url, data=json.dumps(self.es.query()))

    @patch("requests.post")
    def test_swap(self, post_mock):
        self.es.process = Mock()
        self.es.swap()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "swap")
        post_mock.assert_called_with(url, data=json.dumps(self.es.query()))

    @patch("requests.post")
    def test_units(self, post_mock):
        self.es.units()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "cpu_max")
        aggregation = {"units": {"cardinality": {"field": "host.keyword"}}}
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=aggregation)))

    @patch("requests.post")
    def test_requests_min(self, post_mock):
        self.es.requests_min()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "response_time")
        aggregation = {"sum": {"sum": {"field": "count"}}}
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=aggregation)))

    @patch("requests.post")
    def test_response_time(self, post_mock):
        self.es.response_time()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "response_time")
        aggregation = {
            "stats": {"stats": {"field": "value"}},
            "percentiles": {"percentiles": {"field": "value"}}
        }
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=aggregation)))

    @patch("requests.post")
    def test_http_methods(self, post_mock):
        self.es.http_methods()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "response_time")
        aggregation = {"method": {"terms": {"field": "method"}}}
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=aggregation)))

    @patch("requests.post")
    def test_status_code(self, post_mock):
        self.es.status_code()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "response_time")
        aggregation = {"status_code": {"terms": {"field": "status_code"}}}
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=aggregation)))

    @patch("requests.post")
    def test_top_slow(self, post_mock):
        self.es.top_slow()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "response_time")
        query = {
            "query": self.es.filtered_query,
            "size": 0,
            "aggs": {
                "top": {
                    "terms": {
                        "script": "doc['method'].value +'|-o-|'+doc['path.keyword'].value +'|-o-|'+doc['status_code'].value",
                        "order": {
                            "stats.max": "desc"
                        }
                    },
                    "aggs": {
                        "stats": {"stats": {"field": "value"}},
                        "percentiles": {"percentiles": {"field": "value"}},
                        "max": {
                            "top_hits": {
                                "sort": [{"value": {"order": "desc"}}],
                                "size": 1
                            }
                        }
                    }
                }
            }
        }
        post_mock.assert_called_with(url, data=json.dumps(query))

    @patch("requests.post")
    def test_connections(self, post_mock):
        self.es.connections()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "connection")
        aggregation = {"connection": {"terms": {"field": "connection.keyword"}}}
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=aggregation)))

    @patch("requests.post")
    def test_netrx(self, post_mock):
        self.es.netrx()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "netrx")
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=NET_AGGREGATION)))

    @patch("requests.post")
    def test_nettx(self, post_mock):
        self.es.nettx()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "nettx")
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=NET_AGGREGATION)))

    def test_process(self):
        data = {
            "took": 86,
            "timed_out": False,
            "_shards": {
                "total": 266,
                "successful": 266,
                "failed": 0
            },
            "hits": {
                "total": 644073,
                "max_score": 0,
                "hits": []
            },
            "aggregations": {
                "date": {
                    "buckets": [
                        {
                            "key_as_string": "2015-07-21T19:35:00.000Z",
                            "key": 1437507300000,
                            "doc_count": 9,
                            "stats": {
                                "min": 97517568,
                                "max": 97517568,
                                "avg": 97517568
                            }
                        },
                        {
                            "key_as_string": "2015-07-21T19:36:00.000Z",
                            "key": 1437507360000,
                            "doc_count": 9,
                            "stats": {
                                "min": 97517568,
                                "max": 97517568,
                                "avg": 97517568
                            }
                        }
                    ]
                }
            }
        }
        expected = {
            "data": {
                "max": [[1437507300000, 97517568], [1437507360000, 97517568]],
                "min": [[1437507300000, 97517568], [1437507360000, 97517568]],
                "avg": [[1437507300000, 97517568], [1437507360000, 97517568]],
            },
            "min": 97517568,
            "max": 97517569
        }
        d = self.es.process(data)
        self.assertDictEqual(d, expected)

    def test_process_custom_formatter(self):
        data = {
            "took": 86,
            "timed_out": False,
            "_shards": {
                "total": 266,
                "successful": 266,
                "failed": 0
            },
            "hits": {
                "total": 644073,
                "max_score": 0,
                "hits": []
            },
            "aggregations": {
                "date": {
                    "buckets": [
                        {
                            "key_as_string": "2015-07-21T19:35:00.000Z",
                            "key": 1437507300000,
                            "doc_count": 9,
                            "stats": {
                                "min": 97517568,
                                "max": 97517568,
                                "avg": 97517568
                            }
                        },
                        {
                            "key_as_string": "2015-07-21T19:36:00.000Z",
                            "key": 1437507360000,
                            "doc_count": 9,
                            "stats": {
                                "min": 97517568,
                                "max": 97517568,
                                "avg": 97517568
                            }
                        }
                    ]
                }
            }
        }
        expected = {
            "data": {
                "max": [[1437507300000, 93], [1437507360000, 93]],
                "min": [[1437507300000, 93], [1437507360000, 93]],
                "avg": [[1437507300000, 93], [1437507360000, 93]],
            },
            "min": 93,
            "max": 94
        }
        d = self.es.process(data, formatter=lambda x: x / (1024 * 1024))
        self.assertDictEqual(d, expected)

    def test_process_no_aggregation(self):
        data = {
            "took": 1,
            "timed_out": False,
            "_shards": {
                "total": 0,
                "successful": 0,
                "failed": 0
            },
            "hits": {
                "total": 0,
                "max_score": 0,
                "hits": []
            }
        }
        expected = {'data': {}, 'max': 1, 'min': 0}
        d = self.es.process(data, formatter=lambda x: x / (1024 * 1024))
        self.assertDictEqual(d, expected)

    def test_unit_process_no_aggregation(self):
        data = {
            "took": 1,
            "timed_out": False,
            "_shards": {
                "total": 0,
                "successful": 0,
                "failed": 0
            },
            "hits": {
                "total": 0,
                "max_score": 0,
                "hits": []
            }
        }
        expected = {'data': {}, 'max': 1, 'min': 0}
        d = self.es.base_process(data, self.es.units_process)
        self.assertDictEqual(d, expected)

    def test_request_min_process_no_aggregation(self):
        data = {
            "took": 1,
            "timed_out": False,
            "_shards": {
                "total": 0,
                "successful": 0,
                "failed": 0
            },
            "hits": {
                "total": 0,
                "max_score": 0,
                "hits": []
            }
        }
        expected = {'data': {}, 'max': 1, 'min': 0}
        d = self.es.base_process(data, self.es.requests_min_process)
        self.assertDictEqual(d, expected)

    def test_connections_process_no_aggregation(self):
        data = {
            "took": 1,
            "timed_out": False,
            "_shards": {
                "total": 0,
                "successful": 0,
                "failed": 0
            },
            "hits": {
                "total": 0,
                "max_score": 0,
                "hits": []
            }
        }
        expected = {'data': {}, 'max': 1, 'min': 0}
        d = self.es.base_process(data, self.es.connections_process)
        self.assertDictEqual(d, expected)



import json
import httpretty
import mock
import base64
import bson

from django.test import TestCase
from django.test.client import RequestFactory
from django.http import QueryDict

from tsuru_dashboard import settings

from .views import ListEvent, EventInfo, KindList, EventCancel


class ListEventViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/?page=2")
        self.request.session = {"tsuru_token": "admin"}

    def mock_kinds(self):
        url = '{}/events/kinds'.format(settings.TSURU_HOST)
        body = json.dumps([{}] * 1000)
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_use_list_template(self, token_is_valid):
        self.mock_kinds()

        token_is_valid.return_value = True

        url = '{}/events'.format(settings.TSURU_HOST)
        body = json.dumps([{
            "EndTime": u'2016-08-05T16:35:28.946-03:00',
            "StartTime": u'2016-08-05T16:35:28.835-03:00'
        }] * 1000)
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        response = ListEvent.as_view()(self.request)

        self.assertIn("events/list.html", response.template_name)
        self.assertIn('events', response.context_data.keys())
        self.assertEqual('page=3', response.context_data["next"])
        self.assertEqual('', response.context_data["previous"])
        self.assertIn("2", httpretty.last_request().querystring["page"])

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_preserves_querystring_in_pagination(self, token_is_valid):
        self.mock_kinds()

        token_is_valid.return_value = True

        url = '{}/events'.format(settings.TSURU_HOST)
        body = json.dumps([{
            "EndTime": u'2016-08-05T16:35:28.946-03:00',
            "StartTime": u'2016-08-05T16:35:28.835-03:00'
        }] * 1000)
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        self.request = RequestFactory().get("/?page=3&kindName=something&errorOnly=true")
        self.request.session = {"tsuru_token": "admin"}
        response = ListEvent.as_view()(self.request)

        self.assertIn("events/list.html", response.template_name)
        self.assertIn('events', response.context_data.keys())
        next_dict = QueryDict(response.context_data["next"])
        prev_dict = QueryDict(response.context_data["previous"])
        self.assertDictEqual(QueryDict('page=4&kindName=something&errorOnly=true'), next_dict)
        self.assertDictEqual(QueryDict('page=2&kindName=something&errorOnly=true'), prev_dict)
        self.assertIn("3", httpretty.last_request().querystring["page"])

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_kinds(self, token_is_valid):
        self.mock_kinds()

        token_is_valid.return_value = True

        url = '{}/events'.format(settings.TSURU_HOST)
        body = json.dumps([{
            "EndTime": u'2016-08-05T16:35:28.946-03:00',
            "StartTime": u'2016-08-05T16:35:28.835-03:00'
        }] * 1000)
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        response = ListEvent.as_view()(self.request)

        self.assertIn('kinds', response.context_data.keys())

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_return_empty_list_when_status_is_204(self, token_is_valid):
        self.mock_kinds()

        token_is_valid.return_value = True

        url = '{}/events'.format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, url, status=204)

        response = ListEvent.as_view()(self.request)

        self.assertIn("events/list.html", response.template_name)
        self.assertListEqual([], response.context_data['events'])

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_return_empty_kind(self, token_is_valid):
        url = '{}/events/kinds'.format(settings.TSURU_HOST)
        body = json.dumps(range(10000))
        httpretty.register_uri(httpretty.GET, url, body=body, status=204)

        token_is_valid.return_value = True

        url = '{}/events'.format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, url, status=204)

        response = ListEvent.as_view()(self.request)
        self.assertListEqual([], response.context_data['kinds'])

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_filter_by_kind(self, token_is_valid):
        self.mock_kinds()

        token_is_valid.return_value = True

        url = '{}/events?kind='.format(settings.TSURU_HOST)

        def body(request, uri, headers):
            events = []
            if request.querystring.get("kindName", [""])[0] != "nothing":
                events = [{"kind": "some"}]
            return (200, headers, json.dumps(events))

        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        request = RequestFactory().get("")
        request.session = {"tsuru_token": "admin"}
        response = ListEvent.as_view()(request)
        self.assertEqual(len(response.context_data['events']), 1)

        request = RequestFactory().get("?kindName=nothing")
        request.session = {"tsuru_token": "admin"}
        response = ListEvent.as_view()(request)
        self.assertEqual(len(response.context_data['events']), 0)


class EventInfoViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/?page=2")
        self.request.session = {"tsuru_token": "admin"}

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_event_info(self, token_is_valid):
        token_is_valid.return_value = True

        event_id = "abc123"
        url = '{}/events/{}'.format(settings.TSURU_HOST, event_id)
        body = json.dumps({
            "StartCustomData": {
                "Data": base64.b64encode(bson.BSON.encode({"start": 1}))
            },
            "EndCustomData": {
                "Data": base64.b64encode(bson.BSON.encode({"end": 1}))
            },
            "OtherCustomData": {
                "Data": base64.b64encode(bson.BSON.encode({"other": 1}))
            }
        })
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        response = EventInfo.as_view()(self.request, uuid="abc123")

        self.assertIn("events/info.html", response.template_name)
        self.assertIn('event', response.context_data.keys())

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_event_info_not_found(self, token_is_valid):
        token_is_valid.return_value = True

        event_id = "abc123"
        url = '{}/events/{}'.format(settings.TSURU_HOST, event_id)
        httpretty.register_uri(httpretty.GET, url, status=400)

        response = EventInfo.as_view()(self.request, uuid="abc123")

        self.assertIn("events/info.html", response.template_name)
        self.assertIn('event', response.context_data.keys())


class KindListViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/?page=2")
        self.request.session = {"tsuru_token": "admin"}

    def mock_kinds(self):
        url = '{}/events/kinds'.format(settings.TSURU_HOST)
        body = json.dumps([{"Type": "permission", "Name": "app.update.cname.add"}])
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_view(self, token_is_valid):
        self.mock_kinds()

        response = KindList.as_view()(self.request)
        data = json.loads(response.content)
        self.assertEqual(data[0]["Name"], "app.update.cname.add")


class EventCancelViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().post("/", {"reason": "myreason"})
        self.request.session = {"tsuru_token": "admin"}

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_view(self, token_is_valid):
        url = '{}/events/my-uuid/cancel'.format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.POST, url, status=204)

        EventCancel.as_view()(self.request, uuid="my-uuid")

        request = httpretty.last_request()
        self.assertEqual("POST", request.method)
        self.assertEqual("/events/my-uuid/cancel", request.path)
        self.assertEqual("reason=myreason", request.body)



import requests
import bson
import base64

import json
import yaml

from django.views.generic import TemplateView
from django.http import JsonResponse, HttpResponse

from pygments import highlight
from pygments.lexers import YamlLexer
from pygments.formatters import HtmlFormatter

from dateutil import parser

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView


class KindList(LoginRequiredView):

    def get_kinds(self):
        url = '{}/events/kinds'.format(settings.TSURU_HOST)
        response = requests.get(
            url, headers=self.authorization, params=self.request.GET)

        if response.status_code == 204:
            return []

        return response.json()

    def get(self, *args, **kwargs):
        return JsonResponse(self.get_kinds(), safe=False)


class ListEvent(LoginRequiredView, TemplateView):
    template_name = "events/list.html"

    def get_events(self, skip, limit):
        url = '{}/events?skip={}&limit={}'.format(
            settings.TSURU_HOST, skip, limit)
        response = requests.get(
            url, headers=self.authorization, params=self.request.GET)

        if response.status_code == 204:
            return []

        events = response.json()

        for event in events:
            if "StartTime" in event:
                event["StartTime"] = parser.parse(event["StartTime"])
            if "EndTime" in event:
                event["EndTime"] = parser.parse(event["EndTime"])

        return events

    def get_kinds(self):
        url = '{}/events/kinds'.format(settings.TSURU_HOST)
        response = requests.get(
            url, headers=self.authorization, params=self.request.GET)

        if response.status_code == 204:
            return []

        return response.json()

    def get_context_data(self, *args, **kwargs):
        context = super(ListEvent, self).get_context_data(*args, **kwargs)

        page = 1
        try:
            page = int(self.request.GET.get('page', '1'))
        except ValueError:
            pass

        skip = (page - 1) * 20
        limit = 20

        context['events'] = self.get_events(skip, limit)
        context['kinds'] = self.get_kinds()

        req_copy = self.request.GET.copy()
        if len(context['events']) >= 20:
            req_copy['page'] = page + 1
            context['next'] = req_copy.urlencode()

        if page > 1:
            req_copy['page'] = page - 1
            if req_copy['page'] == 1:
                del req_copy['page']
            context['previous'] = req_copy.urlencode()

        return context


def event_serialization_default(obj):
    if hasattr(obj, 'isoformat'):
        return obj.isoformat()
    return None


class EventInfo(LoginRequiredView, TemplateView):
    template_name = "events/info.html"

    def get_event(self, uuid):
        url = '{}/events/{}'.format(settings.TSURU_HOST, uuid)
        response = requests.get(
            url, headers=self.authorization, params=self.request.GET)

        if response.status_code == 200:
            event = response.json()
            return self.decode_custom_data(event)

        return None

    def decode_custom_data(self, event):
        fields = ["StartCustomData", "EndCustomData", "OtherCustomData"]

        for field in fields:
            if event.get(field) and event[field].get("Data"):
                data = self.decode_bson(event[field])
                data = json.loads(json.dumps(data, default=event_serialization_default))
                data = yaml.safe_dump(data, default_flow_style=False, default_style='')
                data = highlight(data, YamlLexer(), HtmlFormatter())
                event[field]["Data"] = data

        return event

    def decode_bson(self, data):
        return bson.BSON(base64.b64decode(data["Data"])).decode()

    def get_context_data(self, *args, **kwargs):
        context = super(EventInfo, self).get_context_data(*args, **kwargs)
        context['event'] = self.get_event(kwargs["uuid"])
        return context


class EventCancel(LoginRequiredView):

    def cancel(self, uuid, reason):
        url = '{}/events/{}/cancel'.format(settings.TSURU_HOST, uuid)
        data = {"reason": reason}
        requests.post(url, data=data, headers=self.authorization)

    def post(self, *args, **kwargs):
        uuid = kwargs["uuid"]
        self.cancel(uuid, self.request.POST.get("reason", ""))
        return HttpResponse()
